/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */



package com.funambol.blackberry.autotest;

/**
 * This interface owns all of the commands available in the automatic test
 * scripting language for Blackberry device.
 */
public interface BlackBerryUserCommands {

    /**
     * Perform a trckwheel motion in the given direction
     * @param direction can be equal to "Up", "Down", "Left", "Right", "Click"
     * @param repeatCount is the number of times this action must be performed
     * @example TrackWheel("Down", 3) generates the trackball down event 3 times
     */
    public static final String TRACK_WHEEL_COMMAND = "TrackWheel";

    /**
     * This command simulates a user entering a value into a field.
     * The field is the one that has the "focus". If the focus is on a field for
     * which it does not make sense to enter text, then the command and the test
     * fail.
     * @param value an arbitrary string to be set in the field. If the field
     *              accepts only one type of characters then the value shall
     *              contain only valid characters. If invalid characters are
     *              supplied the test behavior is unpredictable.
     *              For choice groups, the value represent the index (an integer
     *              starting from 0) of the selected item
     * @example SetFieldText("http://syncserver.somewhere.com/sync")
     */
    public static final String SET_FIELD_TEXT_COMMAND = "SetFieldText";

    /**
     * This command simulates a user clicking on a menu item. The menu is actually opened
     * and the selected voice "clicked".
     * @param item the string shown on the menu item
     * @example ClickMenuItem("Account") click the menu item named "Account"
     */
    public static final String CLICK_MENU_ITEM_COMMAND  = "ClickMenuItem";

    /**
     * This command waits for a certain screen to be displayed and blocks the
     * script execution until such a screen is visible or the given timeout expires.
     *
     * @param name the screen name is a string that can take one of the values
     * listed here below.
     * <ul>
     *   <li> RimAccountScreen is the screen to set the account. This screen is
     *        used for both the login and any further account change</li>
     *   <li> RimHomeScreen is the main screen of the application</li>
     * </ul>
     * @param timeout the timeout is an integer value expressed in milliseconds.
     * @example WaitForScreen(HomeScreen, 20) waits 20 seconds the home screen
     * to be displayed
     */
    public static final String WAIT_FOR_SCREEN_COMMAND  = "WaitForScreen";

    /**
     * This command clears the client configuration. After this command is executed,
     * the client is configuration-less.
     * @example ResetConfig()
     */
    public static final String RESET_CONFIG_COMMAND = "ResetConfig";

    /**
     * This command checks if the name of a source visible in the main screen is
     * a given value.
     * @param idx is the source position (starting from 0 if the SyncAll button
     * is not visible)
     * @param value is the name of the source
     * @example CheckVisibleSourceName(2, Calendar) check if the name of the
     * visible source is "Calendar" ant if it is at pasition 2 (from the top of
     * the screen)
     */
    public static final String CHECK_VISIBLE_SOURCE_NAME_COMMAND = "CheckVisibleSourceName";

    /**
     * Checks if the name of sources count corresponds to a given value.
     * @param count is the expected number of sources in the main screen
     * (including, if visible, the SyncAll meta source).
     * @example CheckVisibleSourceCount(4) checks if there are 4 visible sources
     */
    public static final String CHECK_VISIBLE_SOURCE_COUNT_COMMAND = "CheckVisibleSourceCount";

    /**
     * Don't use this command. It has not been implemented yet.
     */
    public static final String SAVE_ACCOUNT_COMMAND = "SaveAccount";

    /**
     * Check if the given alert message is equal to the given value
     * @param message the String message to be checked
     * @example CheckAlertMessage("alert message")
     */
    public static final String CHECK_ALERT_MESSAGE_COMMAND = "CheckAlertMessage";

    /**
     * This command deletes (resets) all the file in a given directory.
     * @param dir the directory to be reset. Directories in the device memory
     * are named like: file:///store/ while file:///SDCard/ can be used for the
     * memory card.
     * @example ResetDir(file:///store/)
     */
    public static final String RESET_DIR_COMMAND = "ResetDir";

    /**
     * This command copies one file somewhere. File names have the following
     * prefix:
     *
     * <ul>
     *   <li> file:///store/ if they are in the memory of the device </li>
     *   <li> file:///SDCard/ if they are in the sdcard </li>
     * </ul>
     *
     * @param sourceFile the source file name.
     * @param targetFile the target file name.
     * @example CopyFile(file:///store/file.txt, file:///store/newfile.txt)
     */
    public static final String COPY_FILE_COMMAND = "CopyFile";

    /**
     * This command renames one file. File names have the following
     * prefix:
     *
     * <ul>
     *   <li> file:///store/ if they are in the memory of the device </li>
     *   <li> file:///SDCard/ if they are in the sdcard </li>
     * </ul>
     *
     * @param sourceFile the source file name.
     * @param targetFile the target file name.
     * @example RenameFile(file:///store/file.txt, file:///store/newfile.txt)
     */
    public static final String RENAME_FILE_COMMAND = "RenameFile";

    /**
     * This command deletes one file. File names have the following
     * prefix:
     *
     * <ul>
     *   <li> file:///store/ if they are in the memory of the device </li>
     *   <li> file:///SDCard/ if they are in the sdcard </li>
     * </ul>
     *
     * @param file the source file name
     * @example DeleteFile(file:///store/file.txt)
     */
    public static final String DELETE_FILE_COMMAND = "DeleteFile";

    /**
     * This command sets the device timezone to the given value
     * @param tz is the new timezone as available in the UI
     * @example SetTimeZone(SetTimeZone("America/Los_Angeles"));
     */
    public static final String SET_TIMEZONE_COMMAND = "SetTimeZone";

    /**
     * Set the choice for a multiple choice Field
     * @param fieldID is the index of the field to be set
     * @example SetChoiceField(2) set the user selection to the second choice
     * field of a multiple choice item
     */
    public static final String SET_CHOICE_FIELD_COMMAND       = "SetChoiceField";

    /**
     * Set or unset the check on a checkboxfield
     * @param checked is the boolean value to be set
     * @example SetCheckBox(true) put the check on the checkbox
     */
    public static final String SET_CHECK_BOX_COMMAND          = "SetCheckBox";

    /**
     * Set the device date to a given value. The date can be expressed in the
     * following formats:
     *
     * <ul>
     *   <li> yyyyMMddTHHmmssZ </li>
     *   <li> yyyyMMddTHHmmss </li>
     *   <li> yyyyMMdd </li>
     *   <li> yyyy-MM-ddTHH:mm:ssZ </li>
     *   <li> yyyy-MM-ddTHH:mm:ss </li>
     * </ul>
     *
     * @example SetDate(20020602)
     */
    public static final String SET_DATE_TIME_COMMAND          = "SetDateTime";

    /**
     * Writes a string into the currectly focused text field.
     *
     * @example WriteStringWithKeypad(username)
     */
    public static final String WRITE_STRING_WITH_KP_COMMAND   = "WriteStringWithKeypad";
}
