/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;

import com.funambol.blackberry.ui.rim.RimDisplayManager;

import com.funambol.syncml.spds.DeviceConfig;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.client.configuration.Configuration;
import com.funambol.util.AppProperties;
import com.funambol.util.Log;

/**
 * Configuration class for configuration details
 */
public class BBConfiguration extends Configuration {

    private static final String TAG_LOG = "BBConfiguration";

    private static final String CONF_FILE_SYSTEM_USN    = "FILE_SYSTEM_USN";
    private static final String CONF_KEY_C2S_PUSH       = "C2S_PUSH";
    private static final String CONF_NET_USAGE_ACCEPTED = "NETWORK_USAGE_ACCEPTED";

    public static final long CONFIG_KEY = 0xbc565a75eda532eL;

    private long    lastUSN = 0;
    private boolean c2sPush;
    private boolean networkUsageWarningAccepted = false;

    private DeviceConfig devConf = null;
    private ConfigData contents;
    private static BBConfiguration instance;

    public BBConfiguration() {
        super(BBCustomization.getInstance(), BBAppSyncSourceManager.getInstance());
    }

    public static BBConfiguration getInstance() {
        if (instance == null) {
            instance = new BBConfiguration();
        }
        return instance;
    }

    /**
     * This must be called once at the beginning of the application.
     **/
    public void init(Class c) throws Exception {
         if (!initialized) {
             Log.trace(TAG_LOG, "Initializing configuration");

             // Retrieve all the configuration so that from this point it can be
             // written
             final PersistentObject pobj = PersistentStore.getPersistentObject(CONFIG_KEY);
             if (pobj != null) {
                 try {
                     contents = (ConfigData) pobj.getContents();
                 } catch (Exception e) {
                     Log.error(TAG_LOG, "Cannot load configuration", e);
                 }
             }
             if (contents == null) {
                 Log.error(TAG_LOG, "Configuration not set, using default values");
                 contents = new ConfigData();
                 copyDefaults();
             }
         }
         initialized = true;
    }


    public long getLastUSN() {
        return lastUSN;
    }

    public void setLastUSN(long usn) {
        lastUSN = usn;
    }

    public boolean getC2SPush() {
        return c2sPush;
    }

    public void setC2SPush(boolean value) {
        c2sPush = value;
    }

    public void setNetwokUsageWarningAccepted(boolean value) {
        networkUsageWarningAccepted = value;
    }

    public boolean getNetwokUsageWarningAccepted() {
        return networkUsageWarningAccepted;
    }



    public int load() {
        int res = super.load();
        // Now load the custom fields
        lastUSN = loadLongKey(CONF_FILE_SYSTEM_USN, 0);
        c2sPush = loadBooleanKey(CONF_KEY_C2S_PUSH, false);
        networkUsageWarningAccepted = loadBooleanKey(CONF_NET_USAGE_ACCEPTED, false);
        return res;
    }

    public int save() {
        // Save all the custom fields
        saveLongKey(CONF_FILE_SYSTEM_USN, lastUSN);
        saveBooleanKey(CONF_KEY_C2S_PUSH, c2sPush);
        saveBooleanKey(CONF_NET_USAGE_ACCEPTED, networkUsageWarningAccepted);
        // Now save everything
        int res = super.save();
        // On the BB for backward compatible the save performs the commit
        if (res == CONF_OK) {
            commit();
        }
        return res;
    }

    protected void copyDefaults() {
        super.copyDefaults();
        
        Log.info(TAG_LOG, "Copying default configuration values");
       
        // Jad has precedence over Customization as it allows a per user
        // settings
        AppProperties properties = new AppProperties(null);
        String checkIntervalprop = properties.get(AppProperties.AUTOUPDATE_CHECK_INTERVAL);
        String reminderIntervalprop = properties.get(AppProperties.AUTOUPDATE_REMINDER_INTERVAL);

        if(checkIntervalprop == null){
            checkInterval = customization.getCheckUpdtIntervalDefault();
        }else{
            checkInterval = Long.parseLong(checkIntervalprop);
        }
        if(reminderIntervalprop == null){
            reminderInterval = customization.getReminderUpdtIntervalDefault();
        }else{
            reminderInterval = Long.parseLong(reminderIntervalprop);
        }

        c2sPush = false;
        lastUSN = 0;
    }

    protected String getUserAgent() {
        // Remember to update the blackberry synclet pattern (server side) when changing the user agent
        return BBCustomization.getInstance().getUserAgentName();
    }

    /**
     * Helper function to build the DevConf
     * 
     * @return A DeviceConfig
     */
    protected DeviceConfig getDeviceConfig() {
        if (devConf != null) {
            return devConf;
        }
        devConf = new DeviceConfig();
        devConf.man = DeviceInfo.getManufacturerName();
        devConf.mod = DeviceInfo.getDeviceName();
        devConf.swv = BBVersion.VERSION;
        devConf.hwv = "" + DeviceInfo.getPlatformVersion();
        devConf.devID = "fbb-" + DeviceInfo.getDeviceId();
        devConf.setMaxMsgSize(64 * 1024);
        devConf.loSupport = false;
        devConf.utc = true;
        devConf.nocSupport = true;
        return devConf;
    }

    public void saveByteArrayKey(String key, byte[] value) {
        contents.setValue(key, value);
    }

    public byte[] loadByteArrayKey(String key, byte[] defaultValue) {
        return contents.getValue(key, defaultValue);
    }

    public boolean commit() {
        Log.trace(TAG_LOG, "Committing configuration");
        final PersistentObject pobj = PersistentStore.getPersistentObject(CONFIG_KEY);
        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return true;
    }

    protected String loadKey(String key) {
        return contents.getValue(key, (String)null);
    }

    protected void saveKey(String key, String value) {
        if (value == null) {
            contents.setValue(key, "");
        } else {
            contents.setValue(key, value);
        }
    }

    protected void migrateConfig() {
        // Version 7 is 8.7.1 and 8 is 8.7.2 which are the last two versions
        // before we switched to the client API module which were at version 9.
        Log.debug(TAG_LOG, "Migrating config from " + version);

        if ("7".equals(version) || "8".equals(version)) {

            // Constants for sync mode got changed, migrate them
            if (syncMode == 0) {
                syncMode = SYNC_MODE_MANUAL;
            } else if (syncMode == 1) {
                syncMode = SYNC_MODE_SCHEDULED;
            }
            
            // First run timestamp was renamed
            firstRunTimestamp = contents.getValue("FIRST_RUN_TS", 0L);

            // Force a request for server caps
            setForceServerCapsRequest(true);
        } 
        // Now migrate the basic configuration
        super.migrateConfig();
    }
}


