/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.blackberry.controller;

import java.io.IOException;
import java.io.InputStream;
import net.rim.blackberry.api.mail.*;

import com.funambol.blackberry.BBCustomization;

import com.funambol.client.controller.AdvancedSettingsScreenController;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.client.localization.Localization;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.ui.AdvancedSettingsScreen;
import com.funambol.client.ui.Screen;

import com.funambol.platform.FileAdapter;
import com.funambol.util.FileAppender;
import com.funambol.util.Log;

/**
 * Realize the MVC control section of the view elements' in all of the
 * AdvancedSettingsScreen interface implementation. It uses the screen in order
 * to set and get the log level and correctly initialize and dinamically manage
 * its view elements. This class is abstract as it contains generic methods to
 * send and view log; such those methods are implementation dependent as their
 * realization is different on different platforms.
 */
public class BBAdvancedSettingsScreenController extends AdvancedSettingsScreenController {

    /** The tag to be wirtten into the log*/
    public static final String TAG_LOG = "BBAdvancedSettingsScreenController";

    private DialogController dialogController;

    /**
     * public constructor: uses the simple controller object and the controlled
     * screen because on one side it must be compliant with the customization,
     * localization and configuration related to the screen and on the other
     * side it must control the behavior of the screen elements referring to the
     * AdvancedSettingsScreen interface.
     * @param controller the object that contains the customization,
     * localization and configuration instances.
     * @param screen the AdvancedSettingsScreen implementation to be controlled
     * by this class.
     */
    public BBAdvancedSettingsScreenController(com.funambol.client.controller.Controller controller,
            Customization customization, Configuration configuration,
            Localization localization, AppSyncSourceManager appSyncSourceManager,
            AdvancedSettingsScreen screen, DialogController dialogController)
    {
        super(controller, customization, configuration, localization, appSyncSourceManager, screen);
        this.dialogController = dialogController;
    }

    public Screen getAdvancedSettingsScreen() {
        return screen;
    }

    /**
     * Realize the Send log action. As the send log feature is platform
     * dependent this method is left abstract and must be implemented on the
     * high level client controller.
     */
    public void sendLog() {
        Log.debug(TAG_LOG, "Starting thread to send log");
        LogSenderThread lst = new LogSenderThread();
        lst.setPriority(Thread.MIN_PRIORITY);
        lst.start();
    }

    private class LogSenderThread extends Thread {

        public void run() {
            Log.trace(TAG_LOG, "LogSenderThread.run");

            // Prepare an email to send
            FileAppender appender = null;
            try {
                // Give a feedback to the user
                StringBuffer alertMsg = new StringBuffer(localization.getLanguage("sendlog_sending_to"));
                alertMsg.append(" ").append(((BBCustomization)customization).getLogEmailAddress());
                dialogController.showMessage(alertMsg.toString(), 3000);

                Session session = Session.getDefaultInstance();
                if (session == null) {
                    Log.error(TAG_LOG, "Cannot send log because session is null");
                    return;
                }

                Log.trace(TAG_LOG, "Getting transport");
                Transport transport = session.getTransport();

                // Preparing the message
                Store store = Session.getDefaultInstance().getStore();
                Folder[] folders = store.list(Folder.SENT);
                Folder sentfolder = folders[0];
                Message msg = new Message(sentfolder);
                // do not ask for the return recipt
                msg.setFlag(Message.Flag.REQUEST_READ_ACK, false);

                // Prepare the address
                Address addr = new Address(((BBCustomization)customization).getLogEmailAddress(),
                                           ((BBCustomization)customization).getLogEmailName());
                msg.addRecipient(Message.RecipientType.TO, addr);
                msg.setSubject("BlackBerry Sync Client log");

                Multipart mp = new Multipart();
                String mimeType = TextBodyPart.getTextPlainContentType();
                Log.trace(TAG_LOG, "Getting log content");

                //LogContent logContent = Log.getCurrentLogContent();
                BodyPart contentPart;

                // The getContent method is too slow on BB, we need to send the
                // log in a different way
                appender = (FileAppender)Log.getAppender();
                // Disable the log temporary
                appender.closeLogFile();

                // Read the log content
                InputStream is            = null;
                byte        fileContent[] = null;
                String fileName = null;
                FileAdapter file = null;
                try {
                    // Read the log content
                    String logFilePath = customization.getLogFileDirectory();
                    String logFileName = customization.getLogFileName();
                    if (logFilePath.endsWith("/")) {
                        fileName = logFilePath + logFileName;
                    } else {
                        fileName = logFilePath + "/" + logFileName;
                    }
                    file = new FileAdapter(fileName);
                    is = file.openInputStream();
                    fileContent = new byte[(int)file.getSize()];
                    is.read(fileContent);
                } catch (IOException ioe) {
                    Log.error(TAG_LOG, "Cannot read log content file: " + ioe.toString());
                } finally {
                    if (is != null) {
                        try {
                            is.close();
                        } catch (IOException ioe) {
                        }
                        is = null;
                    }
                    if (file != null) {
                        try {
                            file.close();
                        } catch (IOException ioe) {
                        }
                        file = null;
                    }
                }

                SupportedAttachmentPart cp = new SupportedAttachmentPart(mp);
                cp.setContentType(mimeType);
                if (fileContent != null) {
                    cp.setContent(fileContent);
                }
                cp.setFilename(customization.getLogFileName());
                contentPart = cp;
                mp.addBodyPart(contentPart);

                // If we have the rotated log, we shall attach that as well
                if (fileName != null) {
                    try {
                        file = new FileAdapter(fileName + ".sav.txt", true);
                        is = file.openInputStream();
                        fileContent = new byte[(int)file.getSize()];
                        is.read(fileContent);
                        cp = new SupportedAttachmentPart(mp);
                        cp.setContentType(mimeType);
                        if (fileContent != null) {
                            cp.setContent(fileContent);
                        }
                        cp.setFilename(customization.getLogFileName() + ".sav.txt");
                        contentPart = cp;
                        mp.addBodyPart(contentPart);
                    } catch (IOException ioe) {
                        // If we cannot attach it, we just ignore it
                        Log.error(TAG_LOG, "Cannot read log content file: " + ioe.toString());
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException ioe) {
                            }
                            is = null;
                        }
                        if (file != null) {
                            try {
                                file.close();
                            } catch (IOException ioe) {
                            }
                            file = null;
                        }
                    }
                }

                // Reinit the log file
                appender.initLogFile();
                appender = null;

                msg.setContent(mp);

                // Finally send the message
                Log.trace(TAG_LOG, "Sending message");
                transport.send(msg);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot send the log as attachment " + e.toString());
            } finally {
                // Reenable log if it has not been enabled yet
                if (appender != null) {
                    appender.initLogFile();
                }
            }
        }
    }

    /**
     * Realize the View log action. As the view log feature is platform
     * dependent this method is left abstract and must be implemented on the
     * high level client controller.
     */
    public void viewLog() {
    }

    /**
     * Run the Import Contacts process
     */
    public void importContacts() {
    }
}
